// The following code queries and displays information about all
// partitions in the blue gene system.

#include <stdio.h>
#include <stdlib.h>
#include "llapi.h"

const char* partitionStateString(int);
const char* connectionString(int );
const char* nodeMode(int );
const char* switchStateString(int);
const char* dimmString(int dim);

void  displayList(char ** );

main(int argc, char *argv[])
{
	LL_element *queryObject;
	LL_element *machine;
	LL_element  *pr;
	LL_element *swt;
	LL_element *conn;
	const char *partstate;
	const char *partid;
	char *BP_id;
	char *partition_id;
	char *switch_id;
	char *owner;
	char *mloader_image;
	char *blrts_image;
	char *linux_image;
 	char *ram_disk_image;
	char *description;
	char **bp_list;
	char **node_card_list;
	int obj_count;
	int err_code;
	int rc;
	int state;
	int dim;
 	int partition_count;       
 	int bp_count;       
 	int switch_count;       
 	int conn_count;       
	int connection;
	int partition_mode;
	int small_partition;
	int from_port;
	int to_port;


	// Initialize the query blue gene
	queryObject = ll_query(BLUE_GENE);
	if (!queryObject) {
		printf("Query Blue Gene: ll_query() returns NULL.\n");
		 exit(1);
	}

	// Set the query request for all partitions
	char *partition_list[2];
	partition_list[0] = "all";
	partition_list[1] = NULL;
	rc = ll_set_request(queryObject, QUERY_BG_PARTITION, partition_list, ALL_DATA);
	if (rc) {
		printf("Query MACHINES: ll_set_request() return code = %d\n",
		        rc);
		exit(1);
	}

  // Get the blue gene machine object. Only one machine object is returned
	// The blue gene machine object will only contain data for the partitions.
	err_code = 0;
	obj_count = 0;
	machine = ll_get_objs(queryObject, LL_CM, NULL, &obj_count, &err_code);
	if (machine == NULL || obj_count != 1) {
		printf("Query Blue Gene: ll_get_objs() error code = %d\n", err_code);
		exit(1); 
	}


	// Get data from blue gene machine
	rc = ll_get_data(machine, LL_BgMachinePartitionCount, &partition_count);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachinePartitionCount rc = %d\n",rc);
		exit(1);
	}
	printf("The number of Partitions defined in the Blue Gene System = %d.\n", partition_count);


	//
	// Partitions
	//
	rc = ll_get_data(machine, LL_BgMachineGetFirstPartition, &pr);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineGetFirstPartition rc = %d\n",rc);
		exit(1);
	}

	while(pr != NULL) {
		rc = ll_get_data(pr, LL_BgPartitionId, &partition_id);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionId rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(pr, LL_BgPartitionState, &state);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionState rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(pr, LL_BgPartitionBPCount, &bp_count);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionBPCount rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(pr, LL_BgPartitionSwitchCount, &switch_count);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionSwitchCount rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(pr, LL_BgPartitionConnection, &connection);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionConnection rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(pr, LL_BgPartitionOwner, &owner);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionOwner rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(pr, LL_BgPartitionMode, &partition_mode);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionMode rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(pr, LL_BgPartitionSmall, &small_partition);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionSmall rc = %d\n",rc);
			exit(1);
		}

		printf("Partition: id=%s state=%s BPCount=%d SwitchCount=%d Conn=%s User=%s Mode=%s Small=%s\n",
		      partition_id, partitionStateString(state), bp_count,switch_count,connectionString(connection),
			owner, nodeMode(partition_mode), small_partition ? "TRUE" : "FALSE" );

		free(partition_id);
		free(owner);


		rc = ll_get_data(pr, LL_BgPartitionMLoaderImage, &mloader_image);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionMLoaderImage rc = %d\n",rc);
			exit(1);
		}
		printf("\tLoader image file name=%s\n", mloader_image );
		free(mloader_image);

		rc = ll_get_data(pr, LL_BgPartitionBLRTSImage, &blrts_image);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionBLRTSImage rc = %d\n",rc);
			exit(1);
		}
		printf("\tCnode kernel image file name=%s\n", blrts_image );
		free(blrts_image);

		rc = ll_get_data(pr, LL_BgPartitionLinuxImage, &linux_image);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionLinuxImage rc = %d\n",rc);
			exit(1);
		}
		printf("\tI/O nodes linux image file name=%s\n", linux_image );
		free(linux_image);

		rc = ll_get_data(pr, LL_BgPartitionRamDiskImage, &ram_disk_image);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionRamDiskImage rc = %d\n",rc);
			exit(1);
		}
		printf("\tRamdisk image file name=%s\n", ram_disk_image );
		free(ram_disk_image);

		rc = ll_get_data(pr, LL_BgPartitionDescription, &description);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionDescription rc = %d\n",rc);
			exit(1);
		}
		printf("\tDescription=%s\n", description );
		free(description);

		rc = ll_get_data(pr, LL_BgPartitionBPList, &bp_list);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionBPList rc = %d\n",rc);
			exit(1);
		}
		printf("\tBase Partition List = ");
		displayList(bp_list);
		printf("\n");
		fflush(stdout);
		free(bp_list);

		rc = ll_get_data(pr, LL_BgPartitionNodeCardList, &node_card_list);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionNodeCardList rc = %d\n",rc);
			exit(1);
		}
		if (node_card_list) {
			printf("\tNode Card List  = ");
			displayList(node_card_list);
			printf("\n");
			fflush(stdout);
			free(node_card_list);
		}

		// Switches assigned to the partition
		rc = ll_get_data(pr, LL_BgPartitionGetFirstSwitch, &swt);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionGetFirstSwitch rc = %d\n",rc);
			exit(1);
		}
		while(swt != NULL) {
			rc = ll_get_data(swt, LL_BgSwitchId, &switch_id);
			if (rc) {
				printf("Error: ll_get_data LL_BgSwitchId rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(swt, LL_BgSwitchBasePartitionId, &BP_id);
			if (rc) {
				printf("Error: ll_get_data LL_BgSwitchBasePartitionId rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(swt, LL_BgSwitchState, &state);
			if (rc) {
				printf("Error: ll_get_data LL_BgSwitchState rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(swt, LL_BgSwitchDimension, &dim);
			if (rc) {
				printf("Error: ll_get_data LL_BgSwitchDimension rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(swt, LL_BgSwitchConnCount, &conn_count);
			if (rc) {
				printf("Error: ll_get_data LL_BgSwitchConnCount rc = %d\n",rc);
				exit(1);
			}

			printf("\tSwitch: id=%s BP_id=%s state=%-7.7s dim=%s connections=%d\n",
				switch_id, BP_id, switchStateString(state), dimmString(dim), conn_count);
			free(switch_id);
			free(BP_id);

			//
			// Switch Connections
			//
			rc = ll_get_data(swt, LL_BgSwitchGetFirstConn, &conn);
			if (rc) {
				printf("Error: ll_get_data LL_BgSwitchGetFirstConn rc = %d\n",rc);
				exit(1);
			}

			while(conn != NULL) {
				rc = ll_get_data(conn, LL_BgPortConnToSwitchPort, &to_port);
				if (rc) {
					printf("Error: ll_get_data LL_BgPortConnToSwitchPort rc = %d\n",rc);
					exit(1);
				}

				rc = ll_get_data(conn, LL_BgPortConnFromSwitchPort, &from_port);
				if (rc) {
					printf("Error: ll_get_data LL_BgPortConnFromSwitchPort rc = %d\n",rc);
					exit(1);
				}

				printf("\t\tConnection: from=%d to=%d\n", from_port, to_port);

				rc = ll_get_data(swt, LL_BgSwitchGetNextConn, &conn);

				if (rc) {
					printf("Error: ll_get_data LL_BgSwitchGetNextConn rc = %d\n",rc);
					exit(1);
				}
			}

			rc = ll_get_data(pr, LL_BgPartitionGetNextSwitch, &swt);
			if (rc) {
				printf("Error: ll_get_data LL_BgPartitionGetNextSwitch rc = %d\n",rc);
				exit(1);
			}
		}

		rc = ll_get_data(machine, LL_BgMachineGetNextPartition, &pr);
		if (rc) {
			printf("Error: ll_get_data LL_BgMachineGetNextBP rc = %d\n",rc);
			exit(1);
		}
		printf("\n");
	}


	// Free objects obtained from Negotiator
	ll_free_objs(queryObject);
	// Free query element
	ll_deallocate(queryObject);
}

const char* partitionStateString(int state) {
  char* str_state;
  switch ( state ) {
    case BG_PARTITION_FREE:
      str_state = "FREE";
      break;
    case BG_PARTITION_CONFIGURING:
      str_state = "CFG";  
      break;        
    case BG_PARTITION_READY:
      str_state = "READY";
      break;
    case BG_PARTITION_BUSY:
      str_state = "BUSY";
      break;
    case BG_PARTITION_DEALLOCATING:
      str_state = "DEALC";
      break;
    case BG_PARTITION_ERROR:
      str_state = "ERROR";
      break;
    case BG_PARTITION_NAV:
      str_state = "NOT AVAILABLE";
      break;
    default:
      str_state = "<unknown>";
      break;
  }
  return str_state;
}

const char* nodeMode(int m) {
  char* str_mode;
  switch ( m ) {
    case COPROCESSOR:
      str_mode = "COPROCESSOR";
      break;
    case VIRTUAL_NODE:
      str_mode = "VIRTUAL_NODE";
      break;
    default:
      str_mode = "<unknown>";
      break;
  }
  return str_mode;
}

const char* connectionString(int conn) {
  char* str_conn;
  switch ( conn ) {
    case TORUS:
      str_conn = "TORUS";
      break;
    case MESH:
      str_conn = "MESH";
      break;
    case PREFER_TORUS:
      str_conn = "PREFER_TORUS";
      break;
    case BG_NAV:
      str_conn = "NOT AVAILABLE";
      break;
    default:
      str_conn = "<unknown>";
      break;
  }
  return str_conn;
}

void displayList(char **ids)
{
  int i;

  for (i=0; ids[i]; i++ ){ 
		printf("%s  ", ids[i]); 
  }
  return;
}

const char* switchStateString(int state) {
  char* str_state;
  switch ( state ) {
    case BG_SWITCH_UP:
      str_state = "UP";
      break;
    case BG_SWITCH_DOWN:
      str_state = "DOWN";
      break;
    case BG_SWITCH_MISSING:
      str_state = "MISSING";
      break;
    case BG_SWITCH_ERROR:
      str_state = "ERROR";
      break;
    case BG_SWITCH_NAV:
      str_state = "NAV";
      break;
    default:
      str_state = "<unknown>";
      break;
  }
  return str_state;
}

const char* dimmString(int dim) {
  char* str_dim;
  switch ( dim ) {
    case BG_DIM_X:
      str_dim = "X";
      break;
    case BG_DIM_Y:
      str_dim = "Y";
      break;
    case BG_DIM_Z:
      str_dim = "Z";
      break;
    case BG_DIM_NAV:
      str_dim = "NAV";
      break;
    default:
      str_dim = "<unknown>";
      break;
  }
  return str_dim;
}

